@@ -1,6 +1,7 @@
 package Errorlog;
 use strict;
 
+use Time::HiRes qw( gettimeofday tv_interval);
 
 use Baseobject;
 use Exporter;
@@ -13,8 +14,8 @@ use Sys::Syslog;
 use Errorlog::TeeOutput;
 
 
-my $version ='$Id: Errorlog.pm 9128 2005-11-23 09:42:21Z anton $ ';
-our $VERSION="1.18";
+my $version ='$Id: Errorlog.pm 14037 2007-09-12 08:28:37Z anton $ ';
+our $VERSION="1.22";
 #global variable, needed for TeeOutput
 our $SELF;
 
@@ -22,11 +23,11 @@ END
 {
   if ($?)
   {
+    warn "Exitcode: $?";
     if ($SELF->{fields}->{mailonfailure})
     {
-      my $logbuffer="Errolog hint: specify 'logbuffer' as output on new Errorlog () and the errortext will be mailed to you as well.\n";
-      $logbuffer =$SELF->{fields}->{logbuffer} if ($SELF->{fields}->{logbuffer});
-      $SELF->mail("Errorlog failure: Errorstatus: $?, \n$logbuffer") ;
+      my $logbuffer=$SELF->{fields}->{logbuffer} if ($SELF->{fields}->{logbuffer});
+      $SELF->mail("Errorlog v$VERSION failure: Errorstatus: $?, \n$logbuffer") ;
     }
   }
 }
@@ -36,6 +37,9 @@ sub new
 {
   my $pkg=shift;
   my $self= bless (new Baseobject(@_),$pkg);
+  $self->{tstart}= [gettimeofday];
+  $self->{tstored}=$self->{tstart};
+
   $self->field('name','Log');
 
   my $fh;
@@ -70,13 +74,14 @@ sub new
                   'printinfo'=>1,
                   'sysloglevel'=>'info',
                   'syslogmaxbuf'=>512+255, # just guessing, 1024 is too big, things get truncated
+		  'hirestimer'=>0,
                   'user'=>$user,
+		  'version'=>$VERSION,
 		}
 		);
 
   #note: modified version of field is called
   $self->fields(@_);
-
   if ($configdir)
   {
     unless ( -d $configdir )
@@ -90,7 +95,11 @@ sub new
 #  }
 
   $self->overrule();
-  return $self if ($self->error());
+  if ($self->error())
+  {
+    warn "Errorlog failure";
+    return $self ;
+  }
 
   my @params=();
   my %params;
@@ -224,6 +233,9 @@ sub field
 #NOTE NO self
 sub teecallback
 {
+  my $hiRes='';
+  $hiRes = $SELF->hiResTime() if ($SELF->{fields}->{hirestimer});
+
   #NOTE: MUST RETURN TRUE, otherwise nothing is printed!
   my ($fh, $stderr, @string)=@_;
   my $longmess="";
@@ -237,7 +249,7 @@ sub teecallback
     $head = $SELF->leadingstring($fh);
     if ($head)
     {
-      $head="[$SELF->{hostname}:$0] --> $head"  if ($stderr);
+      $head="[$SELF->{hostname}:${0}] --> $head"  if ($stderr);
       syslog($SELF->{fields}->{sysloglevel}, ($head)) ;
     }
     foreach my $string (@string)
@@ -267,12 +279,11 @@ sub teecallback
 
 	else
 	{
-	  syslog($SELF->{fields}->{sysloglevel},"%s",$substr);
+	  syslog($SELF->{fields}->{sysloglevel},"%s","${hiRes}$substr");
 	}
       }
     }
 
-#    warn  "VERBOSITY: ", $Baseobject::verbosity;
     if ($Baseobject::verbosity >= 3)
     {
       $longmess=Errorlog::presentstackframe(' -  ') ;
@@ -300,14 +311,16 @@ sub teecallback
 	push @line,"$line\n";
 	#think polish: leading string last!
 	#add pid to stderror to distinguish this run
+
 	my $leading='';
-	$leading="[$$] [$SELF->{hostname}:$0]     -" if ($SELF->fields->{printinfo});
+	$leading="[$$] [$SELF->{hostname}:$0]  -" if ($SELF->fields->{printinfo});
 	$longmess=Errorlog::presentstackframe($leading) if ($Baseobject::verbosity >= 3);
 	if ($SELF->{fields}->{printinfo})
 	{
-	  unshift (@line, "[$SELF->{hostname}:$0] --> " ) if ($stderr);
+	  unshift (@line, "[$SELF->{hostname}:${0}$hiRes] --> " ) if ($stderr);
 	  unshift (@line, "[$$] ".$SELF->leadingstring($fh));
 	}
+
 	if ($longmess)
 	{
 	  $longmess =~ s/\t/ /g;
@@ -321,6 +334,21 @@ sub teecallback
   1;
 }
 
+
+sub hiResTime
+{
+  my $self=shift;
+  my $tprev=$self->{tstored};
+  $self->{tstored} = [gettimeofday];
+
+  my $tdif=tv_interval ($tprev, $self->{tstored});
+  my $ttotal=tv_interval ($self->{tstart}, $self->{tstored});
+  my $slow='';
+  $slow=" TIME POOL" if ($tdif >= $self->{fields}->{hirestimer});
+
+  return "dt: $tdif tt: ${ttotal}$slow - ";
+}
+
 sub makelogfilename
 {
   my $self=shift;
@@ -351,9 +379,10 @@ sub DESTROY
 sub leadingstring
 {
   my $self=shift;
+  return "" unless  ($self->field('marknewrun') );
+
   my $fh=shift;
   my $delim=' -- ';
-
   my $pid="";
   $pid="\n[$$] "  unless ($fh eq 'syslog');
 
@@ -364,28 +393,23 @@ sub leadingstring
   my $user="";
   $user = "$delim$myuser" if (defined(($myuser)));
 
-
   my $myheader=$self->field('header')||"";
   my $header="";
   $header = "$delim$myheader" if (defined($myheader) and $myheader);
 
-  my $timestamp=scalar(localtime(time()));
-
   my $leadingstring="";
-  if ($self->field('marknewrun') )
+  # print a newrunmarker once ( was: for each filehandle)
+  unless (defined($self->{marked}->{$fh}))
   {
-    # print a newrunmarker once ( was: for each filehandle)
-    unless (defined($self->{marked}->{$fh}))
-    {
-      my $marker=$self->field('newrunmarker');
-      my $script="";
-      $script=$0; $script =~ s/^.*\///; $script.=$delim ;
+    my $timestamp=scalar(localtime(time()));
+    my $marker=$self->field('newrunmarker');
+    my $script="";
+    $script=$0; $script =~ s/^.*\///; $script.=$delim ;
 
-      $leadingstring= "-- $script$timestamp $ipaddress$user$header $marker$pid";
-      $self->{marked}->{$fh}=1;
-    }
+    $leadingstring= "-- $script$timestamp $ipaddress$user$header $marker$pid";
+    $self->{marked}->{$fh}=1;
   }
-  return "$leadingstring";
+  return $leadingstring;
 }
 
 sub mail
@@ -393,7 +417,7 @@ sub mail
   my $self=shift;
   my $string=join ("\n",@_);
 
-  carp "Sending mail to: $self->{fields}->{mailto}" ;
+  warn "Sending mail to: $self->{fields}->{mailto}" ;
   $self->debug ("using mailprog: $self->{fields}->{mailprogram}");
 
   unless ($self->{fields}->{mailto})
@@ -404,13 +428,15 @@ sub mail
 
   my $mailhandle=new FileHandle();
 
-  if (open ( $mailhandle , "| $self->{fields}->{mailprogram} -f $self->{fields}->{mailerrorsto} $self->{fields}->{mailto}" ))
+  if (open ( $mailhandle , "|-", "$self->{fields}->{mailprogram}",
+	     "-f $self->{fields}->{mailerrorsto}",
+	     "$self->{fields}->{mailto}" ))
   {
     my $mail = join ("\n",
 		     (
 		      "To: <$self->{fields}->{mailto}>",
 		      "X-mailer: perl $]",
-		      "Subject: $self->{fields}->{mailsubject}",
+		      "Subject: $self->{fields}->{mailsubject}($self->{fields}->{facility})",
 		      "Precedence: bulk",
 		      "From: $self->{fields}->{mailfrom}",
 		      "Errors-To: $self->{fields}->{mailerrorsto}",
@@ -502,7 +528,6 @@ sub overrule
     foreach $basepart ( 'defaults', $self->basefilename(),'overrule' )
     {
       $configfile="$configdir/${basepart}_errorlog.cf";
-      $self->debug( "trying configfile: $configfile\n");
       next unless ( -r $configfile );
       $self->debug( "found configfile: $configfile\n");
 
@@ -512,7 +537,6 @@ sub overrule
 	return ;
       }
 
-
       local $_;
       while (<$fh>)
       {
